@@ -1,4 +1,3 @@
-import time
 import logging
 import functools
 import json
@@ -18,7 +17,6 @@ from .utils import SimpleJsonEncoder
 from .utils import get_request_data
 from .pack import SimpleJsonResultPacker
 from .pack import SafeJsonResultPacker
-from .models import ApiResponseTimeLog
 
 logger = logging.getLogger(__name__)
 
@@ -63,14 +61,12 @@ class ApiviewRequest(object):
             return funcutils.call_with_inject(func, self.data)
 
 class Apiview(object):
-    def __init__(self, packer, preload_plugins=None, extra_parameters=None, enable_api_response_time_log=None):
+    def __init__(self, packer, preload_plugins=None, extra_parameters=None, **kwargs):
         self.packer = packer
         self.extra_parameters = extra_parameters or {}
         self.preload_plugins = preload_plugins or []
-        if enable_api_response_time_log is None:
-            enable_api_response_time_log = getattr(settings, "DJANGO_APIVIEW_ENABLE_API_RESPONSE_TIME_LOG", False)
-        self.enable_api_response_time_log = enable_api_response_time_log
-    
+        self.apiview_init_kwargs = kwargs
+
     def set_packer(self, packer):
         self.packer = packer
 
@@ -83,21 +79,10 @@ class Apiview(object):
     def put_extra_parameters(self, params):
         self.extra_parameters.update(params)
 
-    def log_api_response_time(self, path, response_time):
-        if self.enable_api_response_time_log:
-            try:
-                log = ApiResponseTimeLog()
-                log.path = path
-                log.response_time = int(response_time)
-                log.save()
-            except Exception:
-                logger.exception("Save api response time log failed.")
-
     def __call__(self, func):
         """Turn the view function into apiview function. Must use as the first decorator.
         """
         def wrapper(request, **kwargs):
-            stime = time.time()
             target_func = func
             defaults = {}
             defaults.update(self.extra_parameters)
@@ -114,16 +99,12 @@ class Apiview(object):
                 if not isinstance(error, bizerror.BizErrorBase):
                     error = bizerror.BizError(error)
                 package = self.packer.pack_error(error, **_django_apiview_request.data)
-            etime = time.time()
-            self.log_api_response_time(_django_apiview_request.request.path, 1000000*(etime-stime))
             return JsonResponse(package, encoder=SimpleJsonEncoder, json_dumps_params={"ensure_ascii": False, "allow_nan": True, "sort_keys": True})
         wrapper.csrf_exempt = True
         return functools.wraps(func)(wrapper)
-        
 
 apiview = Apiview(simple_result_packer)
 
-
 class requires(ApiviewDecorator):
     """Make sure the parameters are given.
     """
@@ -133,10 +114,19 @@ class requires(ApiviewDecorator):
     def main(self, _django_apiview_request, func):
         missing_names = []
         for name in self.parameter_names:
-            if not name in _django_apiview_request.data:
-                missing_names.append(name)
+            if isinstance(name, str):
+                if not name in _django_apiview_request.data:
+                    missing_names.append(name)
+            elif isinstance(name, (list, set, tuple)):
+                flag = False
+                for element in name:
+                    if _django_apiview_request.data.get(element, None):
+                        flag = True
+                        break
+                if not flag:
+                    missing_names.append("({0})".format(" or ".join(name)))
         if missing_names:
-            raise bizerror.MissingParameter(missing_names)
+            raise bizerror.MissingParameter(parameter=",".join(missing_names))
         return super().main(_django_apiview_request, func)
 
 class choices(ApiviewDecorator):
@@ -254,14 +244,14 @@ class cache(ApiviewDecorator):
             eps.sort(key=lambda x:x["entry_point"])
             return eps
 
-    def __call__(self, func):
-        entry_point = ".".join([func.__module__, func.__name__])
+    def keep_related_models_mapping(self):
+        entry_point = id(self)
         self.related_models_mapping[entry_point] = {
             "entry_point": entry_point,
             "related_models": [] + self.related_models,
             "key": self.key,
         }
-        return super().__call__(func)
+        logger.info("Cache entry discovered: {0}".format(self.related_models_mapping[entry_point]))
 
     def __init__(self, key, expire=None, cache_name="default", get_from_cache=None, set_to_cache=True, disable_get_from_cache_header=None, batch_mode=False, ignore_cache_errors=True, related_models=None):
         self.key = key
@@ -274,6 +264,7 @@ class cache(ApiviewDecorator):
         self.batch_mode = batch_mode
         self.ignore_cache_errors = ignore_cache_errors
         self.related_models = related_models or []
+        self.keep_related_models_mapping()
 
     def main(self, _django_apiview_request, func):
         # Import here, so that we don't need django-redis by default.
@@ -439,9 +430,3 @@ class string_length_limit(ApiviewDecorator):
         return super().main(_django_apiview_request, func)
 
 
-
-@apiview
-@check_aclkey()
-def updateApiResponseTimeStats():
-    from .models import ApiResponseTimeStats
-    return ApiResponseTimeStats.update()
